      subroutine magsph(x,eps,mu,numang,theta,
     & qext,qsca,qbac,g,s1,s2)
c
c ****************************************************
c
c Subroutine magsph computes the scattering cross sections and angular
c scattering from a magnetic sphere. If the number of scattering angles
c is set to zero, only the cross sections (efficiencies) are returned.
c
c     Merrill Milham        >>> version 2.0 <<<            SEPT 1993
c
c     Inputs :
c               x = size parameter of the sphere           (real*8)
c             eps = complex permittivity: epsr -i*epsi     (complex*16)
c              mu = complex permeability: mur - i*mui      (complex*16)
c          numang = number of scattering angles            (integer)
c                   between 0 & 90 deg.
c           theta = scattering angles in degrees           (real*8)
c                theta(i) are entered between 0 & 90 deg.
c                theta must increase monotonically. Results for
c                supplementary angles (180 deg. - theta(i)) are
c                also returned.
C
c     Outputs :
c           qext = extinction efficiency
c           qsca = scattering efficiency
c           qbac = backscatter efficiency
c               g = asymmetry factor
c              s1 = scattered amplitude
c              s2 = scattered amplitude
C
c Subroutines used:
c
c           zbjy - returns one-half integer order J & Y Bessel functions
c
c References:
c M. Kerker, D.4. Wang, and C. L. Giles, "Electromagnetic scattering
c by magnetic spheres," J. Opt. Soc. Am., 73, 765-767 (1983).
c
c D. E. Amos, "Algorithm 644:A portable package for Bessel functions of
c a complex argument and nonnegative order," ACM Trans. on Math.
c Software, 12, 265-273 (1986).
c
c M. Abramowitz and I. A. Stegun, "Handbook of Mathematical Functions,"
c NBS Applied Math. Series 55, US Dept. of Commerce, Washington, DC
c (1955).
c
c W. J. Wiscombe,"Mie Scattering Calculations: Advances in Technique
c
c page 25 
c
c and Fast, Vector-Speed Computer Codes," NCAR Tech. Note, NCAR/TN-14O+STR
c (1979)
c
c .........................................................................
c
      implicit none
c
      real*8 x
      complex*16 eps,mu
      integer numang
      real*8 theta(1) 
*
      real*8 qext,qsca,qbac,g
      complex*16 s1(1),s2(1)
c
      integer al,nangl,nangl2
      real*8 third
      parameter (third=1.d0/3.d0,al=5100,nangl=255,nangl2=(nangl+1)/2)
      complex*16 sp(nangl2),sm(nangl2),sps(nangl2),sms(nangl2) 
*
      complex*16 m,mc1,mxi,s,t,u,v,an,bn,xp
      real*8 xi,dn,dnn,rn,tnp1,thetan
      real*8 xmu(nangl),pi(nangl),pil(nangl),tau(nangl) 
*
      real*8 bjr(al),byr(al)
      real*8 cjr(al),cji(al),cyr(al),cyi(al)
      real*8 bjn,byn,bjl
      real*8 sc,ca,t1,t2,t3,t4
      complex*16 cjn,cyn,cjl,cyl,h2n,h2l
      complex*16 anl,bnl,bs,anp,bnp,abp,abm,anpm,bnpm
      complex*16 zt1,zt2 
*
      integer kstop,k,mm,n,j,j2,j3 
*
      real*8 cpi,zero,one,two,rad,half,fnu
      complex*16 cdblei,cdble1,cdble0
      parameter (cpi=3.1415926535897932384d0,zero=0.d0,one=1.d0)
      parameter (two=2.d0,rad=cpi/180.d0,half=0.5d0,fnu=half)
      parameter (cdblei=(0.d0,1.d0),cdble1=(1.d0,0.d0))
      parameter (cdble0=(0.d0,0.d0))
c
      kstop=idint(x+4.d0*x**third+4.d0) 
*
      if(kstop.le.al) then
      continue
                      else
      print*,'magsph arrays too small: kstop =',kstop,' al =',al
      stop
      end if
c
c page 26 
c
      if(numang.eq.0) then
      s1(1)=cdble0
      s2(1)=cdble0
                      else 
*
            if(numang.le.nangl2) then
                continue
                                 else
        print*,numang,'scattering angles input: only',nangl2,' allowed'
            stop
           endif 
*
      do 100 n=1,numang
        
      thetan=dabs(theta(n))
      theta(n)=thetan
*
           if(thetan.le.90.d0) then
               continue
                                   else
      print*,'theta(',n,')=',thetan,'scattering angles must be < 90 deg'
            stop
            end if 
*
      thetan=rad*thetan
      xmu(n)=dcos(thetan) 
*
      sp(n)=cdble0
      sm(n)=cdble0
      sps(n)=cdble0
      sms(n)=cdble0
      pi(n)=half
      pil(n)=zero 
*
100   continue 
*
      end if
c
cthw      m=zsqrt(mu*eps)
      m=sqrt(mu*eps)

      mc1=m/mu 
*
      xi=one/x
      mxi=xi/m
c
      call zbjy(x,m,kstop,fnu,bjr,byr,cjr,cji)
C
      bjl=bjr(1)
      cjl=dcmplx(cjr(1),cji(1))
cthw      cyl=dcmplx(cyr(1),cyi(1))
      h2l=dcmplx(bjr(1),-byr(1))
*
      qext=zero
      qsca=zero
c
c page 27 
c
      bs=cdble0
      g=zero
*
      anl=cdble0
      bnl=cdble0
      dn=one
      rn=one
cthw  tnp1-one 
      tnp1=one
      mm=1
c
      do 300 k=2,kstop
*
      tnp1=tnp1+two
      t1=dn-rn
      ca=one+rn
      sc=rn
*
      dnn=dn+one
      rn=one/dnn
*
      sc=sc+rn
* 
      bjn=bjr(k)
      byn=byr(k)
      cjn=dcmplx(cjr(k),cji(k))
cthw      cyn=dcmplx(cyr(k),cyi(k))
      h2n=dcmplx(bjn,-byn)
*      
      xp=dn*mxi
      s=cjl-xp*cjn
      u=s*h2n
      s=s*bjn
*
      xp=dn*dcmplx(xi,zero)
      t=cjn*(bjl-xp*bjn)
      v=cjn*(h2l-xp*h2n)
*      
      an=(s-mc1*t)/(u-mc1*v)
      bn=(mc1*s-t)/(mc1*u-v)
      abp=an+bn
      abm=an-bn
*
      zt1=dconjg(an)
      zt2=dconjg(bn)
      qext=qext+tnp1*dble(abp)
      qsca=qsca+tnp1*(an*zt1+bn*zt2)
      bs=bs-(dn+half)*mm*abm
      g=g+t1*dble(anl*zt1+bnl*zt2)+sc*dble(an*zt2)
*
      if(numang.eq.0) then
        continue
                      else
c
c page 28 
c
      anp=sc*abp
      bnp=sc*abm
      anpm=mm*anp
      bnpm=mm*bnp
*
      do 375 j=1,numang
      t1=xmu(j)*pi(j)
      t4=t1-pil(j)
      tau(j)=dn*t4-pil(j)
      t2=pi(j)+tau(j)
      t3=pi(j)-tau(j) 
*
      sp(j)=sp(j)+anp*t2
      sms(j)=sms(j)+bnpm*t2
      sm(j)=sm(j)+bnp*t3
      sps(j)=sps(j)+anpm*t3 
*
      pil(j)=pi(j)
      pi(j)=t1+ca*t4 
*
375   continue
      end if 
*
      dn=dnn
      mm=-mm
      anl=an
      bnl=bn 
*
      bjl=bjn
      cjl=cjn
cthw      cyl=cyn
      h2l=h2n 
*
300   continue
C
      if(numang.eq.0) then
      continue
                        else
      	j2=2*numang
      	do 500 j=1,numang
      	j3=j2-j
      	s1(j)=sp(j)+sm(j)
      	s2(j)=sp(j)-sm(j)
      	s1(j3)=sps(j)+sms(j)
      	s2(j3)=sps(j)-sms(j)
500   	continue
*
      end if
c
      xi=two*xi*xi
      qext=xi*qext
      qsca=xi*qsca
c
c page 29
c
      xi=two*xi
      qbac=xi*bs*dconjg(bs)
      g=xi/qsca*g
c
      return
c
	  end